//*********************************************************************************************************************
//*                                                                                                                   *
//*     3delite's Helper functions V1.0 (c) 3delite 2006                                                              *
//*                                                                                                                   *
//* 3delite's Helper functions is free (a mention in the credits would be nice though if you use it!).                *                                            *
//*                                                                                                                   *
//*     http://www.3delite.hu/                                                                                        *
//*                                                                                                                   *
//* If you have any of the aforementioned please email: 3delite@fly.to                                                *
//*                                                                                                                   *
//* Good coding! :)                                                                                                   *
//* 3delite                                                                                                           *
//*********************************************************************************************************************

unit Helper3delite;

interface

Uses
  SysUtils, StrUtils;

    //* Helper functions
    function BGR2RGB(BGRValue: Cardinal): Cardinal;
    function Bytes2MB(Bytes: Int64): String;
    function mSec2Time(mSec: Int64): String;
    function NGetFileSize(FileName: String): Int64;
    function DecodeUnicode(UnicodeString: String): String;

implementation

function BGR2RGB(BGRValue: Cardinal): Cardinal;
begin
    BGRValue := (BGRValue shl 8) shr 8;
   	Result := (BGRValue shr 16) + (((BGRValue shl 16) shr 24) shl 8) + (BGRValue shl 16);
end;

function Bytes2MB(Bytes: Int64): String;
begin
    if Bytes > 1048576 then begin
        if Bytes > 1073741824
            then Result := FloatToStrF((Bytes / 1073741824), ffFixed, 4, 2) + ' GB'
            else Result := FloatToStrF((Bytes / 1048576), ffFixed, 4, 2) + ' MB';
    end else Result := FloatToStrF((Bytes / 1024), ffFixed, 4, 2) + ' KB';
    if Bytes < 1024
 	    then Result := IntToStr(Bytes) + ' Bytes';
end;

function mSec2Time(mSec: Int64): String;
var
  tHours: String;
  tMinutes: String;
  tSecs: String;
  tmSecs: String;
  Seconds: Int64;
begin
    if mSec = 4294967295
        then mSec := 0;
    Seconds := mSec div 1000;
    tHours := IntToStr(Seconds div 3600);
    if Length(tHours) = 1
        then tHours := '0' + tHours;
    tMinutes := IntToStr(((Seconds - (StrToInt(tHours) * 3600)) div 60));
    if Length(tMinutes) = 1
        then tMinutes := '0' + tMinutes;
    tSecs := IntToStr(Seconds - (StrToInt(tHours) * 3600 + StrToInt(tMinutes) * 60));
    if Length(tSecs) = 1
        then tSecs := '0' + tSecs;
    tmSecs := IntToStr(mSec - ((StrToInt(tHours) * 3600 + StrToInt(tMinutes) * 60) + StrToInt(tSecs)) * 1000);
    case Length(tmSecs) of
        1: tmSecs := '00' + tmSecs;
        2: tmSecs := '0' + tmSecs;
    end;
    tmSecs := Copy(tmSecs, 1, 2);
    Result := tHours + '.' + tMinutes + ':' + tSecs + '.' + tmSecs;
end;

function NGetFileSize(FileName: String): Int64;
var
  f: TSearchRec;
begin
    Result := -1;
 	if FileExists(FileName) then begin
    	try
        	if FindFirst(FileName, faAnyFile, f) = 0 then begin
            	Result := f.Size;
            end;
        finally
          	FindClose(f);
        end;
	end;
end;

function DecodeUnicode(UnicodeString: String): String;
begin
    Result := StringReplace(UnicodeString, '&amp;', '&', [rfReplaceAll]);
    Result := StringReplace(Result, '&#32;', ' ', [rfReplaceAll]);
    Result := StringReplace(Result, '&lt;', '<', [rfReplaceAll]);
    Result := StringReplace(Result, '&gt;', '>', [rfReplaceAll]);
    Result := StringReplace(Result, '&quot;', '"', [rfReplaceAll]);
    Result := StringReplace(Result, '&nbsp;', ' ', [rfReplaceAll]);
    Result := StringReplace(Result, '&#162;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&#163;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&#166;', '|', [rfReplaceAll]);
    Result := StringReplace(Result, '&#167;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&copy;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&#171;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&#187;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&reg;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&#177;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&#180;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&#181;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&#164;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&#165;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&#182;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&#183;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&#189;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Aacute;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Atilde;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Auml;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&AElig;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Ccedil;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Eacute;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Ecirc;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Euml;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Iacute;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Icirc;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&ETH;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Ntilde;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Ograve;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Oacute;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Ocirc;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Otilde;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Ouml;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&#215;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Oslash;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Ugrave;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Uacute;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Ucirc;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Uuml;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Uuml;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Yacute;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&szlig;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&aacute;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&acirc;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&atilde;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&auml;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&aring;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&aelig;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&ccedil;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&egrave;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&eacute;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&ecirc;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&euml;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&igrave;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&iacute;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&icirc;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&iuml;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&icirc;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&eth;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&ntilde;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&ograve;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&oacute;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&ocirc;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&otilde;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&ouml;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&#247;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&oslash;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&ugrave;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&uacute;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&ucirc;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&uuml;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&yacute;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&thorn;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&yuml;', '', [rfReplaceAll]);
end;

end.
