unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Helper3delite, SelPath, IniFiles;

type
  TForm1 = class(TForm)
    ButtonTest: TButton;
    OpenDialog1: TOpenDialog;
    GroupBox1: TGroupBox;
    EditBufferSize: TEdit;
    Label1: TLabel;
    Button2: TButton;
    Label2: TLabel;
    EditTestFileName: TEdit;
    ButtonFileSelect: TButton;
    Label3: TLabel;
    EditTempPath: TEdit;
    Button4: TButton;
    ButtonBufferSizeWholeFile: TButton;
    GroupBox2: TGroupBox;
    MemoResults: TMemo;
    CheckBoxWriteTest: TCheckBox;
    EditPasses: TEdit;
    Label4: TLabel;
    EditRawSize: TEdit;
    Label5: TLabel;
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure MemoResultsChange(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure ButtonTestClick(Sender: TObject);
    procedure TestFile(FileName, TempFileName: String);
    procedure ButtonBufferSizeWholeFileClick(Sender: TObject);
    procedure ButtonFileSelectClick(Sender: TObject);
    procedure Button2Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.TestFile(FileName, TempFileName: String);
var
  F, FTemp: File of Byte;
  FileStream, TempStream: TStream;
  Buffer: Array of Byte;
  BufferSize: Integer;
  BytesRead, BytesWrite: Integer;
  PerfFreq, FreqCountStart, FreqCountEnd: Int64;
  ProcessingTimes: Array of Extended;
  MBPerSecs: Array of Extended;
  ProcessingTime: Extended;
  MBPerSec: Extended;
  i, Passes: Cardinal;
  WriteTestFileSize: Int64;
begin

    BufferSize := StrToIntDef(EditBufferSize.Text, 1024);
    Passes := StrToInt(EditPasses.Text);

    SetLength(Buffer, BufferSize);
    SetLength(ProcessingTimes, Passes);
    SetLength(MBPerSecs, Passes);

    MemoResults.Lines.Add('Test File size: ' + bytes2MB(NGetFileSize(FileName)));
    MemoResults.Lines.Add('Buffer size: ' + bytes2MB(BufferSize));

    //* TFileStream
    for i := 0 to Passes - 1 do begin
        QueryPerformanceFrequency(PerfFreq);
        QueryPerformanceCounter(FreqCountStart);

        FileStream := TFileStream.Create(FileName, fmOpenRead);
        try
            repeat
                FileStream.Read(Pointer(Buffer)^, BufferSize);
            until FileStream.Position >= FileStream.Size;
        finally
            FileStream.Free;
        end;

        QueryPerformanceCounter(FreqCountEnd);
        ProcessingTimes[i] := (FreqCountEnd - FreqCountStart) / PerfFreq * 1000;
        MBPerSecs[i] := NGetFileSize(FileName) / ProcessingTimes[i] / 1000;
        MemoResults.Lines.Add('TFileStream read ' + IntToStr(i + 1)  + ': ' + msec2Time(Round(ProcessingTimes[i])) + ' (' + FloatToStrF(MBPerSecs[i], ffFixed, 4, 2) + 'MB/s)');

        Application.ProcessMessages;

    end;

    ProcessingTime := 0;
    MBPerSec := 0;

    for i := 0 to Passes - 1 do begin
        ProcessingTime := ProcessingTime + ProcessingTimes[i];
        MBPerSec := MBPerSec + MBPerSecs[i];
    end;

    ProcessingTime := ProcessingTime / Passes;
    MBPerSec := MBPerSec / Passes;
    MemoResults.Lines.Add('TFileStream read average: ' + msec2Time(Round(ProcessingTime)) + ' (' + FloatToStrF(MBPerSec, ffFixed, 4, 2) + 'MB/s)');

    Application.ProcessMessages;

    //* AssignFile()
    for i := 0 to Passes - 1 do begin
        QueryPerformanceFrequency(PerfFreq);
        QueryPerformanceCounter(FreqCountStart);

        FileMode := fmOpenRead;
        AssignFile(F, FileName);
        Reset(F);
        if IOResult <> 0 then begin
            MemoResults.Lines.Add('AssignFile(): IOError.');
            Exit;
        end;
        try
            repeat
                //* Delphi 2006
                BlockRead(F, Pointer(Buffer)^, BufferSize, BytesRead);
                //* Delphi 7?
                // BlockRead(F, Buffer[0], BufferSize, BytesRead) ;
            until BytesRead = 0 ;
        finally
            CloseFile(F);
        end;

        QueryPerformanceCounter(FreqCountEnd);
        ProcessingTimes[i] := (FreqCountEnd - FreqCountStart) / PerfFreq * 1000;
        MBPerSecs[i] := NGetFileSize(FileName) / ProcessingTimes[i] / 1000;
        MemoResults.Lines.Add('AssignFile() read ' + IntToStr(i + 1)  + ': ' + msec2Time(Round(ProcessingTimes[i])) + ' (' + FloatToStrF(MBPerSecs[i], ffFixed, 4, 2) + 'MB/s)');
    end;

    ProcessingTime := 0;
    MBPerSec := 0;

    for i := 0 to Passes - 1 do begin
        ProcessingTime := ProcessingTime + ProcessingTimes[i];
        MBPerSec := MBPerSec + MBPerSecs[i];
    end;

    ProcessingTime := ProcessingTime / Passes;
    MBPerSec := MBPerSec / Passes;
    MemoResults.Lines.Add('AssignFile() read average: ' + msec2Time(Round(ProcessingTime)) + ' (' + FloatToStrF(MBPerSec, ffFixed, 4, 2) + 'MB/s)');

    Application.ProcessMessages;

    //* Write test
    if CheckBoxWriteTest.Checked then begin
        //* TFileStream
        for i := 0 to Passes - 1 do begin
            QueryPerformanceFrequency(PerfFreq);
            QueryPerformanceCounter(FreqCountStart);

            FileStream := TFileStream.Create(FileName, fmOpenRead);
            TempStream := TFileStream.Create(TempFileName, fmCreate);
            try
                repeat
                    FileStream.Read(Pointer(Buffer)^, BufferSize);
                    TempStream.Write(Pointer(Buffer)^, BufferSize);
                until FileStream.Position >= FileStream.Size;
            finally
                FileStream.Free;
                TempStream.Free;
            end;

            QueryPerformanceCounter(FreqCountEnd);
            ProcessingTimes[i] := (FreqCountEnd - FreqCountStart) / PerfFreq * 1000;
            MBPerSecs[i] := NGetFileSize(TempFileName) / ProcessingTimes[i] / 1000;
            MemoResults.Lines.Add('TFileStream read-write ' + IntToStr(i + 1)  + ': ' + msec2Time(Round(ProcessingTimes[i])) + ' (' + FloatToStrF(MBPerSecs[i], ffFixed, 4, 2) + 'MB/s)');

            DeleteFile(EditTempPath.Text + 'write_test.tmp');

            Application.ProcessMessages;
        end;

        ProcessingTime := 0;
        MBPerSec := 0;

        for i := 0 to Passes - 1 do begin
            ProcessingTime := ProcessingTime + ProcessingTimes[i];
            MBPerSec := MBPerSec + MBPerSecs[i];
        end;

        ProcessingTime := ProcessingTime / Passes;
        MBPerSec := MBPerSec / Passes;
        MemoResults.Lines.Add('TFileStream read-write average: ' + msec2Time(Round(ProcessingTime)) + ' (' + FloatToStrF(MBPerSec, ffFixed, 4, 2) + 'MB/s)');
        Application.ProcessMessages;
  
        //* AssignFile()
        for i := 0 to Passes - 1 do begin
            QueryPerformanceFrequency(PerfFreq);
            QueryPerformanceCounter(FreqCountStart);

            FileMode := fmOpenRead;
            AssignFile(F, FileName);
            AssignFile(FTemp, TempFileName);
            Reset(F);
            Rewrite(FTemp);
            if IOResult <> 0 then begin
                MemoResults.Lines.Add('AssignFile(): IOError.');
                Exit;
            end;
            try
                BytesRead := 0;
                repeat
                    //* Delphi 2006
                    BlockRead(F, Pointer(Buffer)^, BufferSize, BytesRead);
                    BlockWrite(FTemp, Pointer(Buffer)^, BytesRead, BytesWrite);
                    //* Delphi 7?
                    // BlockRead(F, Buffer[0], BufferSize, BytesRead);
                    // BlockWrite(FTemp, Buffer[0], BytesRead, BytesWrite);
                until BytesRead < BufferSize;
            finally
                CloseFile(F);
                CloseFile(FTemp);
            end;

            QueryPerformanceCounter(FreqCountEnd);
            ProcessingTimes[i] := (FreqCountEnd - FreqCountStart) / PerfFreq * 1000;
            MBPerSecs[i] := NGetFileSize(TempFileName) / ProcessingTimes[i] / 1000;
            MemoResults.Lines.Add('AssignFile() read-write ' + IntToStr(i + 1)  + ': ' + msec2Time(Round(ProcessingTimes[i])) + ' (' + FloatToStrF(MBPerSecs[i], ffFixed, 4, 2) + 'MB/s)');

            DeleteFile(EditTempPath.Text + 'write_test.tmp');
        end;

        ProcessingTime := 0;
        MBPerSec := 0;

        for i := 0 to Passes - 1 do begin
            ProcessingTime := ProcessingTime + ProcessingTimes[i];
            MBPerSec := MBPerSec + MBPerSecs[i];
        end;

        ProcessingTime := ProcessingTime / Passes;
        MBPerSec := MBPerSec / Passes;
        MemoResults.Lines.Add('AssignFile() read-write average: ' + msec2Time(Round(ProcessingTime)) + ' (' + FloatToStrF(MBPerSec, ffFixed, 4, 2) + 'MB/s)');

        Application.ProcessMessages;

        //* Random write test
        WriteTestFileSize := StrToInt(EditRawSize.Text) * 1024 * 1024;
        Randomize;
        for i := 0 to BufferSize - 1
            do Buffer[i] := Random(255);

        MemoResults.Lines.Add('Write Test File size: ' + bytes2MB(WriteTestFileSize));

        //* TFileStream
        for i := 0 to Passes - 1 do begin
            QueryPerformanceFrequency(PerfFreq);
            QueryPerformanceCounter(FreqCountStart);

            TempStream := TFileStream.Create(TempFileName, fmCreate);
            try
                repeat
                    TempStream.Write(Pointer(Buffer)^, BufferSize);
                until TempStream.Position >= WriteTestFileSize;
            finally
                TempStream.Free;
            end;

            QueryPerformanceCounter(FreqCountEnd);
            ProcessingTimes[i] := (FreqCountEnd - FreqCountStart) / PerfFreq * 1000;
            MBPerSecs[i] := NGetFileSize(TempFileName) / ProcessingTimes[i] / 1000;
            MemoResults.Lines.Add('TFileStream write ' + IntToStr(i + 1)  + ': ' + msec2Time(Round(ProcessingTimes[i])) + ' (' + FloatToStrF(MBPerSecs[i], ffFixed, 4, 2) + 'MB/s)');

            DeleteFile(EditTempPath.Text + 'write_test.tmp');

            Application.ProcessMessages;
        end;

        ProcessingTime := 0;
        MBPerSec := 0;

        for i := 0 to Passes - 1 do begin
            ProcessingTime := ProcessingTime + ProcessingTimes[i];
            MBPerSec := MBPerSec + MBPerSecs[i];
        end;

        ProcessingTime := ProcessingTime / Passes;
        MBPerSec := MBPerSec / Passes;
        MemoResults.Lines.Add('TFileStream write average: ' + msec2Time(Round(ProcessingTime)) + ' (' + FloatToStrF(MBPerSec, ffFixed, 4, 2) + 'MB/s)');
        Application.ProcessMessages;
  
        //* AssignFile()
        for i := 0 to Passes - 1 do begin
            QueryPerformanceFrequency(PerfFreq);
            QueryPerformanceCounter(FreqCountStart);

            AssignFile(FTemp, TempFileName);
            Rewrite(FTemp);
            if IOResult <> 0 then begin
                MemoResults.Lines.Add('AssignFile(): IOError.');
                Exit;
            end;
            try
                BytesRead := BufferSize;
                BytesWrite := 0;
                repeat
                    //* Delphi 2006
                    BlockWrite(FTemp, Pointer(Buffer)^, BytesRead, BytesWrite);
                    BytesWrite := BytesWrite + BytesRead;
                    //* Delphi 7?
                    // BlockRead(F, Buffer[0], BufferSize, BytesRead);
                    // BlockWrite(FTemp, Buffer[0], BytesRead, BytesWrite);
                until BytesWrite >= WriteTestFileSize;
            finally
                CloseFile(FTemp);
            end;

            QueryPerformanceCounter(FreqCountEnd);
            ProcessingTimes[i] := (FreqCountEnd - FreqCountStart) / PerfFreq * 1000;
            MBPerSecs[i] := NGetFileSize(TempFileName) / ProcessingTimes[i] / 1000;
            MemoResults.Lines.Add('AssignFile() write ' + IntToStr(i + 1)  + ': ' + msec2Time(Round(ProcessingTimes[i])) + ' (' + FloatToStrF(MBPerSecs[i], ffFixed, 4, 2) + 'MB/s)');

            DeleteFile(EditTempPath.Text + 'write_test.tmp');
        end;

        ProcessingTime := 0;
        MBPerSec := 0;

        for i := 0 to Passes - 1 do begin
            ProcessingTime := ProcessingTime + ProcessingTimes[i];
            MBPerSec := MBPerSec + MBPerSecs[i];
        end;

        ProcessingTime := ProcessingTime / Passes;
        MBPerSec := MBPerSec / Passes;
        MemoResults.Lines.Add('AssignFile() write average: ' + msec2Time(Round(ProcessingTime)) + ' (' + FloatToStrF(MBPerSec, ffFixed, 4, 2) + 'MB/s)');

        Application.ProcessMessages;
    end;

    SetLength(Buffer, 0);
    SetLength(ProcessingTimes, 0);
    SetLength(MBPerSecs, 0);
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
    Close;
end;

procedure TForm1.Button4Click(Sender: TObject);
begin
    FormSelPath.EditPath.Text := EditTempPath.Text;
    FormSelPath.ShowModal;
    EditTempPath.Text := FormSelPath.EditPath.Text;
end;

procedure TForm1.ButtonBufferSizeWholeFileClick(Sender: TObject);
begin
    EditBufferSize.Text := IntToStr(NGetFileSize(EditTestFileName.Text));
end;

procedure TForm1.ButtonFileSelectClick(Sender: TObject);
begin
    if OpenDialog1.Execute
        then EditTestFileName.Text := OpenDialog1.FileName;
end;

procedure TForm1.ButtonTestClick(Sender: TObject);
begin
    TestFile(EditTestFileName.Text, EditTempPath.Text + 'write_test.tmp');
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  Ini: TIniFile;
begin
    Ini := TIniFile.Create(ChangeFileExt(Application.ExeName, '.ini'));
    EditBufferSize.Text := Ini.ReadString('Settings', 'BufferSize', '1024');
    EditTestFileName.Text := Ini.ReadString('Settings', 'TestFile', 'C:\');
    EditTempPath.Text := Ini.ReadString('Settings', 'TempPath', 'C:\');
    EditPasses.Text := Ini.ReadString('Settings', 'Passes', '10');
    EditRawSize.Text := Ini.ReadString('Settings', 'RawSize', '500');
    Ini.Free;
end;

procedure TForm1.FormDestroy(Sender: TObject);
var
  Ini: TIniFile;
begin
    Ini := TIniFile.Create(ChangeFileExt(Application.ExeName, '.ini'));
    Ini.WriteString('Settings', 'BufferSize', EditBufferSize.Text);
    Ini.WriteString('Settings', 'TestFile', EditTestFileName.Text);
    Ini.WriteString('Settings', 'TempPath', EditTempPath.Text);
    Ini.WriteString('Settings', 'Passes', EditPasses.Text);
    Ini.WriteString('Settings', 'RawSize', EditRawSize.Text);
    Ini.Free;
end;

procedure TForm1.MemoResultsChange(Sender: TObject);
begin
    MemoResults.Lines.SaveToFile(ExtractFilePath(Application.ExeName) + 'Log.txt');
end;

end.
